﻿using System.Collections;
using System.Collections.Generic;
using System.Linq;

namespace BPA.KitChen.StoreManagement.Application.BaseDto
{
    /// <summary>
    /// 树基类
    /// </summary>
    public interface ITreeNode
    {
        /// <summary>
        /// 获取节点id
        /// </summary>
        /// <returns></returns>
        string GetId();

        /// <summary>
        /// 获取节点父id
        /// </summary>
        /// <returns></returns>
        string GetPid();

        /// <summary>
        /// 设置Children
        /// </summary>
        /// <param name="children"></param>
        void SetChildren(IList children);
    }

    /// <summary>
    /// 递归工具类，用于遍历有父子关系的节点，例如菜单树，字典树等等
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public class TreeBuildUtil<T> where T : ITreeNode
    {
        /// <summary>
        /// 顶级节点的父节点Id(默认00000000-0000-0000-0000-000000000000)
        /// </summary>
        private readonly string _rootParentId = "0";

        /// <summary>
        /// 构造树节点
        /// </summary>
        /// <param name="nodes"></param>
        /// <returns></returns>
        public List<T> Build(List<T> nodes)
        {
            nodes.ForEach(u => BuildChildNodes(nodes, u, new List<T>()));

            var result = new List<T>();
            nodes.ForEach(u =>
            {
                if (_rootParentId == u.GetPid())
                    result.Add(u);
            });
            return result;
        }

        /// <summary>
        /// 构造子节点集合
        /// </summary>
        /// <param name="totalNodes"></param>
        /// <param name="node"></param>
        /// <param name="childNodeList"></param>
        private void BuildChildNodes(List<T> totalNodes, T node, List<T> childNodeList)
        {
            var nodeSubList = new List<T>();
            totalNodes.ForEach(u =>
            {
                if (u.GetPid().Equals(node.GetId()))
                    nodeSubList.Add(u);
            });
            nodeSubList.ForEach(u => BuildChildNodes(totalNodes, u, new List<T>()));
            childNodeList.AddRange(nodeSubList);
            node.SetChildren(childNodeList);
        }
    }


    /// <summary>
    /// 构建树形结构类
    /// </summary>
    public class TreeModel
    {
        public string ID { set; get; }
        public string PARENTID { set; get; }
        public string NAME { set; get; }
        public List<TreeModel> TREECHILDREN { set; get; }
    }
    public class setTreelist
    {
        /// <summary>
        /// 公用递归(反射转换List)
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="allList">数据列表</param>
        /// <param name="parentId">父级ID</param>
        /// <param name="idField">id字段名称</param>
        /// <param name="parentIdField">父级id字段名称</param>
        /// <param name="nameField">name字段名称</param>
        /// <returns></returns>
        public List<TreeModel> ConversionList<T>(List<T> allList, string parentId, string idField, string parentIdField, string nameField)
        {
            List<TreeModel> list = new List<TreeModel>();
            TreeModel model = null;
            foreach (var item in allList)
            {
                model = new TreeModel();
                foreach (System.Reflection.PropertyInfo p in item.GetType().GetProperties())
                {
                    if (p.Name == idField)
                    {
                        model.ID = p.GetValue(item).ToString();
                    }
                    if (p.Name == parentIdField)
                    {
                        model.PARENTID = p.GetValue(item).ToString();
                    }
                    if (p.Name == nameField)
                    {
                        model.NAME = p.GetValue(item).ToString();
                    }
                }
                list.Add(model);
            }
            return OperationParentData(list, parentId);
        }

        /// <summary>
        /// 公用递归(处理递归最父级数据)
        /// </summary>
        /// <param name="treeDataList">树形列表数据</param>
        /// <param name="parentId">父级Id</param>
        /// <returns></returns>
        public List<TreeModel> OperationParentData(List<TreeModel> treeDataList, string parentId)
        {
            var data = treeDataList.Where(x => x.PARENTID == parentId);
            List<TreeModel> list = new List<TreeModel>();
            foreach (var item in data)
            {
                OperationChildData(treeDataList, item);
                list.Add(item);
            }
            return list;
        }
        /// <summary>
        /// 公用递归(递归子级数据)
        /// </summary>
        /// <param name="treeDataList">树形列表数据</param>
        /// <param name="parentItem">父级model</param>
        public void OperationChildData(List<TreeModel> treeDataList, TreeModel parentItem)
        {
            var subItems = treeDataList.Where(ee => ee.PARENTID == parentItem.ID).ToList();
            if (subItems.Count != 0)
            {
                parentItem.TREECHILDREN = new List<TreeModel>();
                parentItem.TREECHILDREN.AddRange(subItems);
                foreach (var subItem in subItems)
                {
                    OperationChildData(treeDataList, subItem);
                }
            }
        }

    }

}